
type term =
  | Var of string
  | Abs of string * term
  | App of term * term

let string_of_term t =
  let rec abs t =
    match t with
      | Abs (s, t') ->
        "lam " ^ lambda t
      | _ -> app t
  and lambda t =
    match t with
      | Abs (s, t') ->
        s ^ " " ^ lambda t'
      | _ -> ". " ^ app t
  and app t =
    match t with
      | App (t1, t2) ->
        app t1 ^ " " ^ var t2
      | _ -> var t
  and var t =
    match t with
      | Var s -> s
      | _ -> "(" ^ abs t ^ ")" in
  abs t

let rec contains x xs =
  match xs with
    | [] -> false
    | x' :: xs ->
      x = x' || contains x xs

let rec equal_term t1 t2 vars =
  match t1, t2 with
    | Var s1, Var s2 -> contains (s1, s2) vars
    | Abs (s1, t1'), Abs (s2, t2') -> 
      equal_term t1' t2' ((s1, s2) :: vars)
    | App (t11, t12), App (t21, t22) ->
      equal_term t11 t21 vars && equal_term t12 t22 vars
    | _ -> false

exception Not_found of string
let rec lookup key map =
  match map with
    | [] -> raise (Not_found key)
    | (k, v) :: map ->
      if k = key
      then v
      else lookup key map

type top_level =
  | Let of string * term
  | Compute of term



