open Ast

let uniq s env =
  let rec loop s env' = 
    match env' with
      | [] -> s
      | (s', _) :: env' ->
        if s = s'
        then loop (s ^ "'") env
        else loop s env' in
  loop s env

let rec inline t env =
  match t with
    | Var s -> lookup s env
    | Abs (s, t) ->
      let s' = uniq s env in
      Abs (s', inline t ((s, Var s') :: env))
    | App (t1, t2) ->
      let t2' = inline t2 env in
      let t1' = inline t1 env in
      App (t1', t2')

let rec eval t env =
  match t with
    | Var s ->
      begin match lookup s env with
        | Var s' when s = s' -> Var s
        | t -> eval t env
      end
    | Abs (s, t) ->
      let s' = uniq s env in
      Abs (s', inline t ((s, Var s') :: env))
    | App (t1, t2) ->
      let t1' = eval t1 env in
      match t1' with
        | Abs (s, t1'') -> 
          eval t1'' ((s, inline t2 env) :: env)
        | _ -> App (t1', eval t2 env)

let rec force t env =
  match eval t env with
    | Abs (s, t) ->
      let s' = uniq s env in
      Abs (s', force t ((s, Var s') :: env))
    | t -> t

let rec interpret ts env =
  match ts with
    | [] -> []
    | Let (s, t) :: ts ->
      interpret ts ((s, eval t env) :: env)
    | Compute t :: ts ->
      print_string (string_of_term t);
      let res = force t env in
      print_string " = ";
      print_string (string_of_term res);
      let rec loop env =
        match env with
          | [] -> ()
          | (s, v) :: env ->
            if equal_term v res [] && not (Var s = t)
            then begin
              print_string " = ";
              print_string s;
              loop env
            end
            else loop env in
      loop env;
      print_newline ();
      interpret ts env












