{
  open Parser
}

let ht = '\t'
let lf = '\n'
let ff = '\012'
let cr = '\r'
let sp = ' '
let line_terminator = lf | cr | cr lf 

rule token = parse
  | eof    { EOF }
(* Whitespace *)
  | (sp | ht | ff)                 { token lexbuf }  (* white_space *)
  | line_terminator                 { Lexing.new_line lexbuf; token lexbuf }
  | "lam"  { LAM }
  | "Let"  { LET }
  | "Compute" { COMPUTE }
  | '.'    { DOT }
  | '='    { EQUALS }
  | '('    { LPAREN }
  | ')'    { RPAREN }
  | "//"       { linecomment lexbuf } 
  | "/*"		   { blockcomment 0 lexbuf }
  | [^ '(' ')' '/' ' ' '\t' '\n' '\012' '\r']+ as s { ID s }
and linecomment = parse
  | (lf | cr) { token lexbuf }
  | _ { linecomment lexbuf }
  | eof { EOF }
and blockcomment level = parse
  | "/*" { blockcomment (level + 1) lexbuf }
  | "*/" { if level = 0 
           then token lexbuf 
           else blockcomment (level - 1) lexbuf }
  | _ { blockcomment level lexbuf }
  | eof { EOF }
